
################################################################
# This is a generated script based on design: AXV7_OV7670_HDMI
#
# Though there are limitations about the generated script,
# the main purpose of this utility is to make learning
# IP Integrator Tcl commands easier.
################################################################

namespace eval _tcl {
proc get_script_folder {} {
   set script_path [file normalize [info script]]
   set script_folder [file dirname $script_path]
   return $script_folder
}
}
variable script_folder
set script_folder [_tcl::get_script_folder]

################################################################
# Check if script is running in correct Vivado version.
################################################################
set scripts_vivado_version 2018.3
set current_vivado_version [version -short]

if { [string first $scripts_vivado_version $current_vivado_version] == -1 } {
   puts ""
   catch {common::send_msg_id "BD_TCL-109" "ERROR" "This script was generated using Vivado <$scripts_vivado_version> and is being run in <$current_vivado_version> of Vivado. Please run the script in Vivado <$scripts_vivado_version> then open the design in Vivado <$current_vivado_version>. Upgrade the design by running \"Tools => Report => Report IP Status...\", then run write_bd_tcl to create an updated script."}

   return 1
}

################################################################
# START
################################################################

# To test this script, run the following commands from Vivado Tcl console:
# source AXV7_OV7670_HDMI_script.tcl


# The design that will be created by this Tcl script contains the following 
# module references:
# camara_ram, controlador_camara, interfaz_i2c, pll_hdmi_1080p, vga_output, vga_to_hdmi

# Please add the sources of those modules before sourcing this Tcl script.

# If there is no project opened, this script will create a
# project, but make sure you do not have an existing project
# <./myproj/project_1.xpr> in the current working folder.

set list_projs [get_projects -quiet]
if { $list_projs eq "" } {
   create_project project_1 myproj -part xc7a100tcsg324-2
}


# CHANGE DESIGN NAME HERE
variable design_name
set design_name AXV7_OV7670_HDMI

# If you do not already have an existing IP Integrator design open,
# you can create a design using the following command:
#    create_bd_design $design_name

# Creating design if needed
set errMsg ""
set nRet 0

set cur_design [current_bd_design -quiet]
set list_cells [get_bd_cells -quiet]

if { ${design_name} eq "" } {
   # USE CASES:
   #    1) Design_name not set

   set errMsg "Please set the variable <design_name> to a non-empty value."
   set nRet 1

} elseif { ${cur_design} ne "" && ${list_cells} eq "" } {
   # USE CASES:
   #    2): Current design opened AND is empty AND names same.
   #    3): Current design opened AND is empty AND names diff; design_name NOT in project.
   #    4): Current design opened AND is empty AND names diff; design_name exists in project.

   if { $cur_design ne $design_name } {
      common::send_msg_id "BD_TCL-001" "INFO" "Changing value of <design_name> from <$design_name> to <$cur_design> since current design is empty."
      set design_name [get_property NAME $cur_design]
   }
   common::send_msg_id "BD_TCL-002" "INFO" "Constructing design in IPI design <$cur_design>..."

} elseif { ${cur_design} ne "" && $list_cells ne "" && $cur_design eq $design_name } {
   # USE CASES:
   #    5) Current design opened AND has components AND same names.

   set errMsg "Design <$design_name> already exists in your project, please set the variable <design_name> to another value."
   set nRet 1
} elseif { [get_files -quiet ${design_name}.bd] ne "" } {
   # USE CASES: 
   #    6) Current opened design, has components, but diff names, design_name exists in project.
   #    7) No opened design, design_name exists in project.

   set errMsg "Design <$design_name> already exists in your project, please set the variable <design_name> to another value."
   set nRet 2

} else {
   # USE CASES:
   #    8) No opened design, design_name not in project.
   #    9) Current opened design, has components, but diff names, design_name not in project.

   common::send_msg_id "BD_TCL-003" "INFO" "Currently there is no design <$design_name> in project, so creating one..."

   create_bd_design $design_name

   common::send_msg_id "BD_TCL-004" "INFO" "Making design <$design_name> as current_bd_design."
   current_bd_design $design_name

}

common::send_msg_id "BD_TCL-005" "INFO" "Currently the variable <design_name> is equal to \"$design_name\"."

if { $nRet != 0 } {
   catch {common::send_msg_id "BD_TCL-114" "ERROR" $errMsg}
   return $nRet
}

##################################################################
# DESIGN PROCs
##################################################################



# Procedure to create entire design; Provide argument to make
# procedure reusable. If parentCell is "", will use root.
proc create_root_design { parentCell } {

  variable script_folder
  variable design_name

  if { $parentCell eq "" } {
     set parentCell [get_bd_cells /]
  }

  # Get object for parentCell
  set parentObj [get_bd_cells $parentCell]
  if { $parentObj == "" } {
     catch {common::send_msg_id "BD_TCL-100" "ERROR" "Unable to find parent cell <$parentCell>!"}
     return
  }

  # Make sure parentObj is hier blk
  set parentType [get_property TYPE $parentObj]
  if { $parentType ne "hier" } {
     catch {common::send_msg_id "BD_TCL-101" "ERROR" "Parent <$parentObj> has TYPE = <$parentType>. Expected to be <hier>."}
     return
  }

  # Save current instance; Restore later
  set oldCurInst [current_bd_instance .]

  # Set parent object as current
  current_bd_instance $parentObj


  # Create interface ports

  # Create ports
  set cam_datos [ create_bd_port -dir I -from 7 -to 0 cam_datos ]
  set cam_href [ create_bd_port -dir I cam_href ]
  set cam_pclk [ create_bd_port -dir I cam_pclk ]
  set cam_pwdn [ create_bd_port -dir O cam_pwdn ]
  set cam_reset [ create_bd_port -dir O -type rst cam_reset ]
  set cam_scl [ create_bd_port -dir O cam_scl ]
  set cam_sda [ create_bd_port -dir O cam_sda ]
  set cam_vsync [ create_bd_port -dir I cam_vsync ]
  set cam_xclk [ create_bd_port -dir O cam_xclk ]
  set clk [ create_bd_port -dir I -type clk clk ]
  set_property -dict [ list \
   CONFIG.FREQ_HZ {100000000} \
 ] $clk
  set hdmi_tx_clk_n [ create_bd_port -dir O -type clk hdmi_tx_clk_n ]
  set hdmi_tx_clk_p [ create_bd_port -dir O -type clk hdmi_tx_clk_p ]
  set hdmi_tx_n [ create_bd_port -dir O -from 2 -to 0 hdmi_tx_n ]
  set hdmi_tx_p [ create_bd_port -dir O -from 2 -to 0 hdmi_tx_p ]
  set reset [ create_bd_port -dir I -type rst reset ]

  # Create instance: camara_ram_0, and set properties
  set block_name camara_ram
  set block_cell_name camara_ram_0
  if { [catch {set camara_ram_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $camara_ram_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create instance: controlador_camara_0, and set properties
  set block_name controlador_camara
  set block_cell_name controlador_camara_0
  if { [catch {set controlador_camara_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $controlador_camara_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create instance: interfaz_i2c_0, and set properties
  set block_name interfaz_i2c
  set block_cell_name interfaz_i2c_0
  if { [catch {set interfaz_i2c_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $interfaz_i2c_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create instance: pll_hdmi_1080p_0, and set properties
  set block_name pll_hdmi_1080p
  set block_cell_name pll_hdmi_1080p_0
  if { [catch {set pll_hdmi_1080p_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $pll_hdmi_1080p_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create instance: vga_output_0, and set properties
  set block_name vga_output
  set block_cell_name vga_output_0
  if { [catch {set vga_output_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $vga_output_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create instance: vga_to_hdmi_0, and set properties
  set block_name vga_to_hdmi
  set block_cell_name vga_to_hdmi_0
  if { [catch {set vga_to_hdmi_0 [create_bd_cell -type module -reference $block_name $block_cell_name] } errmsg] } {
     catch {common::send_msg_id "BD_TCL-105" "ERROR" "Unable to add referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   } elseif { $vga_to_hdmi_0 eq "" } {
     catch {common::send_msg_id "BD_TCL-106" "ERROR" "Unable to referenced block <$block_name>. Please add the files for ${block_name}'s definition into the project."}
     return 1
   }
  
  # Create port connections
  connect_bd_net -net cam_datos_1 [get_bd_ports cam_datos] [get_bd_pins controlador_camara_0/cam_datos]
  connect_bd_net -net cam_href_1 [get_bd_ports cam_href] [get_bd_pins controlador_camara_0/cam_href]
  connect_bd_net -net cam_pclk_1 [get_bd_ports cam_pclk] [get_bd_pins controlador_camara_0/cam_pclk]
  connect_bd_net -net cam_vsync_1 [get_bd_ports cam_vsync] [get_bd_pins controlador_camara_0/cam_vsync]
  connect_bd_net -net camara_ram_0_data_rd [get_bd_pins camara_ram_0/data_rd] [get_bd_pins vga_output_0/vram_dat]
  connect_bd_net -net clk_1 [get_bd_ports clk] [get_bd_pins controlador_camara_0/clk] [get_bd_pins interfaz_i2c_0/clk] [get_bd_pins pll_hdmi_1080p_0/clk]
  connect_bd_net -net controlador_camara_0_cam_pwdn [get_bd_ports cam_pwdn] [get_bd_pins controlador_camara_0/cam_pwdn]
  connect_bd_net -net controlador_camara_0_cam_reset [get_bd_ports cam_reset] [get_bd_pins controlador_camara_0/cam_reset]
  connect_bd_net -net controlador_camara_0_cam_xclk [get_bd_ports cam_xclk] [get_bd_pins controlador_camara_0/cam_xclk]
  connect_bd_net -net controlador_camara_0_vram_clk [get_bd_pins camara_ram_0/clk_wr] [get_bd_pins controlador_camara_0/vram_clk]
  connect_bd_net -net controlador_camara_0_vram_dat [get_bd_pins camara_ram_0/data_wr] [get_bd_pins controlador_camara_0/vram_dat]
  connect_bd_net -net controlador_camara_0_vram_dir [get_bd_pins camara_ram_0/address_wr] [get_bd_pins controlador_camara_0/vram_dir]
  connect_bd_net -net interfaz_i2c_0_cam_scl [get_bd_ports cam_scl] [get_bd_pins interfaz_i2c_0/cam_scl]
  connect_bd_net -net interfaz_i2c_0_cam_sda [get_bd_ports cam_sda] [get_bd_pins interfaz_i2c_0/cam_sda]
  connect_bd_net -net pll_hdmi_1080p_0_clk_pixel_x1 [get_bd_pins pll_hdmi_1080p_0/clk_pixel_x1] [get_bd_pins vga_output_0/clk] [get_bd_pins vga_to_hdmi_0/pixel_clk]
  connect_bd_net -net pll_hdmi_1080p_0_clk_pixel_x5 [get_bd_pins pll_hdmi_1080p_0/clk_pixel_x5] [get_bd_pins vga_to_hdmi_0/pixel_clk_x5]
  connect_bd_net -net pll_hdmi_1080p_0_hdmi_reset [get_bd_pins pll_hdmi_1080p_0/hdmi_reset] [get_bd_pins vga_to_hdmi_0/reset]
  connect_bd_net -net reset_1 [get_bd_ports reset] [get_bd_pins interfaz_i2c_0/reset]
  connect_bd_net -net vga_output_0_vga_blank [get_bd_pins vga_output_0/vga_blank] [get_bd_pins vga_to_hdmi_0/vga_blank]
  connect_bd_net -net vga_output_0_vga_blue [get_bd_pins vga_output_0/vga_blue] [get_bd_pins vga_to_hdmi_0/vga_blue]
  connect_bd_net -net vga_output_0_vga_green [get_bd_pins vga_output_0/vga_green] [get_bd_pins vga_to_hdmi_0/vga_green]
  connect_bd_net -net vga_output_0_vga_hsync [get_bd_pins vga_output_0/vga_hsync] [get_bd_pins vga_to_hdmi_0/vga_hsync]
  connect_bd_net -net vga_output_0_vga_red [get_bd_pins vga_output_0/vga_red] [get_bd_pins vga_to_hdmi_0/vga_red]
  connect_bd_net -net vga_output_0_vga_vsync [get_bd_pins vga_output_0/vga_vsync] [get_bd_pins vga_to_hdmi_0/vga_vsync]
  connect_bd_net -net vga_output_0_vram_clk [get_bd_pins camara_ram_0/clk_rd] [get_bd_pins vga_output_0/vram_clk]
  connect_bd_net -net vga_output_0_vram_dir [get_bd_pins camara_ram_0/address_rd] [get_bd_pins vga_output_0/vram_dir]
  connect_bd_net -net vga_to_hdmi_0_hdmi_tx_clk_n [get_bd_ports hdmi_tx_clk_n] [get_bd_pins vga_to_hdmi_0/hdmi_tx_clk_n]
  connect_bd_net -net vga_to_hdmi_0_hdmi_tx_clk_p [get_bd_ports hdmi_tx_clk_p] [get_bd_pins vga_to_hdmi_0/hdmi_tx_clk_p]
  connect_bd_net -net vga_to_hdmi_0_hdmi_tx_n [get_bd_ports hdmi_tx_n] [get_bd_pins vga_to_hdmi_0/hdmi_tx_n]
  connect_bd_net -net vga_to_hdmi_0_hdmi_tx_p [get_bd_ports hdmi_tx_p] [get_bd_pins vga_to_hdmi_0/hdmi_tx_p]

  # Create address segments


  # Restore current instance
  current_bd_instance $oldCurInst

  validate_bd_design
  save_bd_design
}
# End of create_root_design()


##################################################################
# MAIN FLOW
##################################################################

create_root_design ""


