----------------------------------------------------------------------------------
-- COPYRIGHT 2019 Victor Ral Arteaga Prez
--This program is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.
--
--This program is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--
-- Description: Cdigo VHDL para la captura de datos de la camara OV7670 y almacenamiento en VRAM
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity controlador_camara is
     port(
            clk: in std_logic;
            cam_xclk    : out std_logic;
            cam_href    : in std_logic;
            cam_vsync   : in std_logic;
            cam_pclk    : in std_logic;
            cam_pwdn    : out std_logic; 
            cam_reset   : out std_logic;
            cam_datos   : in std_logic_vector(7 downto 0);
            vram_dat    : out std_logic_vector(15 downto 0);
            vram_dir    : out std_logic_vector(17 downto 0);
            vram_clk    : out std_logic
          );
end controlador_camara;

architecture Behavioral of controlador_camara is
signal pasos: integer range 0 to 1 := 0;
signal direccion: unsigned(vram_dir'length - 1 downto 0) := (others => '1');
begin

cam_pwdn <= '0';
cam_reset <= '1';

process(clk)
variable contador: integer range 0 to 3;
variable clk_out: std_logic;
begin

    if rising_edge(clk) then
    
        contador := contador + 1;
                   
            if (contador < 2) then
                clk_out:='0';
            else
                clk_out:='1';
            end if;
            
            if (contador = 3) then
                contador :=0;
            end if;
            
        end if;
        cam_xclk <= clk_out;
    end process;

   process(cam_pclk,cam_vsync,cam_href)
	variable temporal: std_logic_vector(15 downto 0)  := (others => '0');
   begin
         if cam_vsync = '1' then 
            direccion <= (others => '1');
            pasos <= 0;
         elsif rising_edge(cam_pclk) and cam_href = '1' then     
			
				   if pasos = 0 then
					  temporal(7 downto 0) := cam_datos;
				      vram_clk <= '1'; 
               end if;
				
               if pasos = 1  then
					  temporal(15 downto 8) := cam_datos;
                 direccion <= direccion + 1;
					  vram_clk <= '0';
               end if;

               pasos <= pasos + 1;
         
         vram_dat <= temporal;
         
      end if;
   end process;
	
vram_dir <= std_logic_vector(direccion);
end Behavioral;
