----------------------------------------------------------------------------------
-- COPYRIGHT 2019 Victor Ral Arteaga Prez
--This program is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.
--
--This program is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--
-- Description: Cdigo VHDL para inicializar la camara OV7670 por i2c
-- se recomienda resetear manualmente la configuracin de la camara
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity interfaz_i2c is
    port(
        clk,reset           : in std_logic;
        cam_sda,cam_scl     : out std_logic
    );
end interfaz_i2c;

architecture Behavioral of interfaz_i2c is

TYPE matriz IS ARRAY (0 to 101) OF STD_LOGIC_VECTOR(7 DOWNTO 0);
type state is (paso_1,paso_2,paso_3,paso_4,paso_5,paso_6,paso_7,paso_8,paso_9,paso_10);
signal pr_state, nx_state: state;
signal clk_ms,cambio_matriz,bus_clk,data_clk: std_logic;
signal index: integer range 0 to 102 :=0;
signal timer: integer range 0 to 8;
shared variable bits: integer range 0 to 7;
constant direccion: std_logic_vector(7 downto 0) := X"42";
constant secuencia: matriz := (
X"12",X"04",X"8c",X"00",X"04",X"00",X"40",X"10",X"14",X"28",X"15",X"20",X"11",X"C0",
X"6b",X"0a",X"0c",X"00",
X"32",X"09",--href
X"17",X"14",--hstart
X"18",X"02",--hstop
X"19",X"03",--vstart
X"1a",X"5D",--vstop
X"03",X"00",--vref
X"3e",X"00",X"73",X"f0",X"a2",X"02",X"3b",X"12",X"13",X"e7",X"69",X"00",X"b0",X"84",
X"43",X"0a",X"44",X"f0",X"45",X"34",X"46",X"58",X"47",X"28",X"48",X"3a",X"59",X"88",
X"5a",X"88",X"5b",X"44",X"5c",X"67",X"5d",X"49",X"5e",X"0e",X"6c",X"0a",X"6d",X"55",
X"6e",X"11",X"6f",X"9f",X"6a",X"40",X"01",X"40",X"02",X"40",X"4f",X"80",X"50",X"80",
X"51",X"00",X"52",X"22",X"53",X"5e",X"54",X"80",X"58",X"9e",X"41",X"08",X"09",X"00",
X"55",X"00");
begin
	
process(clk)
variable contador: integer range 0 to 250;
variable clk_out: std_logic;
begin

    if rising_edge(clk) then
    
        contador := contador + 1;
        
        if(contador=250) then
            contador :=0;
        end if;
        
        if (contador<250/2) then
            clk_out:='0';
        else
            clk_out:='1';
        end if;
        
    end if;
    
    clk_ms <= clk_out;

end process;

process(clk_ms)
variable cuenta: integer range 0 to 3;
begin

 if rising_edge(clk_ms) then
 
     cuenta := cuenta + 1;
 
	 if cuenta = 0 then
	   bus_clk <= '0';
	 elsif cuenta = 1 then
	   data_clk <= '1';
	 elsif cuenta = 2 then
	   bus_clk <= '1';
	 else
	   data_clk <= '0';
	 end if;
 
 end if;
end process;

process(data_clk,reset)
begin
 if(reset='1') then
    pr_state <= paso_1;
    bits := 0;
 elsif(data_clk'event and data_clk='1')then
	 if(bits = timer-1) then
         pr_state <=nx_state;
         bits := 0;
	 else
    	 bits := bits + 1;
	 end if;
 end if;
end process;

process(pr_state,index,data_clk,bus_clk,cambio_matriz)
begin
 case pr_state is
 
  when paso_1 => 
  cam_scl <= '1';
  cam_sda <= '1';
  timer <= 1;
  cambio_matriz <= '0';
  nx_state <= paso_2;
  
  when paso_2 => --condicion start
  cam_scl <= '1';
  cam_sda <= data_clk;
  timer <= 1;
  cambio_matriz <= '0';
  nx_state <= paso_3;
  
  when paso_3 =>--escribir direccion
  cam_scl <= bus_clk;
  cam_sda <= direccion(7-bits);
  timer <= 8;
  cambio_matriz <= '0';
  nx_state <= paso_4;
  
  when paso_4 =>
  cam_scl <= bus_clk;
  cam_sda <= 'Z';
  timer <= 1;
  cambio_matriz <= '0';
  nx_state <= paso_5;
  
  when paso_5 => -- escribir registro
  cam_scl <= bus_clk;
  cam_sda <= secuencia(index)(7-bits);
  timer <= 8;
  cambio_matriz <= '0';
  nx_state <= paso_6;
  
  when paso_6 => 
  cam_scl <= bus_clk;
  cam_sda <= 'Z';
  timer <= 1;
  cambio_matriz <= '1';
  nx_state <= paso_7;
  
  when paso_7 => --escribir dato
  cam_scl <= bus_clk;
  cam_sda <= secuencia(index)(7-bits);
  timer <= 8;
  cambio_matriz <= '0';
  nx_state <= paso_8;
  
  when paso_8 => 
  cam_scl <= bus_clk;
  cam_sda <= 'Z';
  timer <= 1;
  cambio_matriz <= '0';
  nx_state <= paso_9;
  
  when paso_9 => --siguiente dato de la matriz "secuencia"
  cam_scl <= '1';
  cam_sda <= '0';
  timer <= 1;
  cambio_matriz <= '1';
  nx_state <= paso_10;
  
  when paso_10 => --reviso si ya acabo de mandar los datos de la matriz "secuencia"
  cam_scl <= '1';
  cam_sda <= '1';
  timer <= 1;
  cambio_matriz <= '0';
  
  if(index < 101) then
    nx_state <= paso_1;
  else
    nx_state <= paso_10;
  end if;
  
 end case;
end process;

process(data_clk,cambio_matriz,reset)
variable valor: integer range 0 to 102 := 0;
begin

if(reset = '1') then
	valor := 0;
elsif rising_edge(data_clk) and cambio_matriz = '1' then
	valor := valor + 1;
end if;

index <= valor;
end process;

end Behavioral;

