----------------------------------------------------------------------------------
-- COPYRIGHT 2019 Victor Ral Arteaga Prez
--This program is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.
--
--This program is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--
-- Description: Cdigo VHDL para la generacin de timings VGA 1920 x 1080 @ 60 Hz 
-- y lectura de memoria VRAM con linear upscaler x3
-- 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity vga_output is
    Port ( 
            clk : in STD_LOGIC;
            vga_hsync   : out std_logic;
            vga_vsync   : out std_logic;
            vga_red     : out std_logic_vector(7 downto 0);
            vga_green   : out std_logic_vector(7 downto 0);
            vga_blue    : out std_logic_vector(7 downto 0);
            vga_blank   : out std_logic;
            vram_dir    : out std_logic_vector(17 downto 0);
            vram_dat    : in std_logic_vector(15 downto 0);
            vram_clk    : out std_logic
    );
end vga_output;

architecture Behavioral of vga_output is

    constant hfront: integer := 88;
    constant hsync: integer := 44;
    constant hvisible: integer := 1920;
    constant hwhole: integer := 2200;
    constant vfront: integer := 4;
    constant vsync: integer := 5;
    constant vvisible: integer := 1080;
    constant vwhole: integer := 1125;

    signal x : unsigned(11 downto 0) := (others => '0');
    signal y : unsigned(11 downto 0) := (others => '0');
    signal xscale: integer range 0 to 4 := 0;
    signal yscale: integer range 0 to 4 := 0;
    signal hcount: integer range 0 to 640 := 0;

begin

timing_gen: process(clk)
    begin
        if rising_edge(clk) then
        
            if x = hvisible then
                vga_blank <= '1';
            elsif x = hwhole - 1 and (y < vvisible - 1 or y = vwhole - 1) then
                vga_blank <= '0';            
            end if;
            
            if x = hvisible + hfront - 1 then
                vga_hsync <= '1';
            elsif x = hvisible + hfront + hsync - 1 then
                vga_hsync <= '0';
            end if;

            if x = hwhole - 1 then 
            
                x <= (others => '0');
                
                    if y = vvisible + vfront - 1 then
                        vga_vsync  <= '1';
                    elsif y = vvisible + vfront + vsync - 1 then
                        vga_vsync  <= '0';
                    end if;
                    
                    if y = vwhole - 1 then
                        y <= (others => '0');
                    else
                        y <= y +1;
                    end if;
            else
                x <= x + 1;        
            end if;     
        end if;                   
    end process;

data_gen: process(clk)

        constant escala : integer := 3;
        constant hw : integer := 640;
        constant vw : integer := 360;
        variable contador_ram: unsigned(vram_dir'length - 1 downto 0) := (others => '0');
        
    begin
 
        vram_clk <= clk;
        
        if rising_edge(clk) then
        
          if (x >= 0 and x < hvisible) and (y >= 0 and  y < vvisible) then   

            xscale <= xscale + 1;
	
                if(xscale = escala - 1) then
                            
                    contador_ram := contador_ram + 1;
                    hcount <= hcount + 1;	
                    xscale <= 0;
                    
                    if(hcount = hw - 1) then
                
                        if(yscale >= 0 and yscale < escala - 1)then
                            yscale <= yscale + 1;
                            contador_ram := contador_ram - hw;
                        else
                        yscale <= 0;
                            if(contador_ram = hw*vw) then
                                contador_ram := (others => '0');
                            end if;	
                        end if;
                        
                        hcount <= 0;
                    end if;			
                    
                end if;                    
                    
                vga_red <= vram_dat(15 downto 11) & "111";
                vga_green <= vram_dat(10 downto 5) & "11";
                vga_blue <= vram_dat(4 downto 0) & "111";
                vram_dir <= std_logic_vector(contador_ram);
                
            else
            
                if (x >= hvisible) or (y >= vvisible) then
                    vga_red    <= (others => '0');
                    vga_green  <= (others => '0');
                    vga_blue   <= (others => '0');  
                else
                    vga_red    <= X"FF";
                    vga_green  <= X"FF";
                    vga_blue   <= X"FF"; 
                end if;
             
            end if;
        end if;
    end process;

end Behavioral;

